package com.sf.javase.day18;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.condition.OS;


import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * 缓冲流
 */
public class T {
    /**
     * 构造器
     */
    @Test
    public void t1() throws FileNotFoundException {
        //  字节缓存流
        BufferedInputStream inputStream = new BufferedInputStream(new FileInputStream("d:\\a.txt"));
        BufferedReader bufferedReader = new BufferedReader(new FileReader("d:\\a.txt"));
    }

    @Test
    public void t2() throws IOException {
        //  普通方式的读写
        long startTime = System.currentTimeMillis();
        File file = new File("D:\\ruanjianZIP\\eclipse-jee-2023-03-R-win32-x86_64.zip");
        FileInputStream fis = new FileInputStream(file);
        FileOutputStream fos = new FileOutputStream(new File("D:\\ruanjianZIP\\eclipse1111-jee-2023-03-R-win32-x86_64.zip"));
        int data;
        byte[] bytes = new byte[1024];
        while ((data = fis.read(bytes)) != -1){
            fos.write(bytes,0,data);
        }
        long endTime = System.currentTimeMillis();
        System.out.println("总计用时"+(endTime - startTime));   //总计用时4961
        fis.close();
        fos.close();
    }


    @Test
    public void t3() throws IOException {
        //  用缓冲区实现复制
        long startTime = System.currentTimeMillis();
        File file = new File("D:\\ruanjianZIP\\eclipse-jee-2023-03-R-win32-x86_64.zip");
        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file));
        BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream("D:\\ruanjianZIP\\eclipse-jee-2023-03-11R-win32-x86_64.zip"));
        int data;
        byte[] bytes = new byte[1024];
        while ((data = bis.read(bytes)) != -1){
            bos.write(bytes,0,data);
        }
        long endTime = System.currentTimeMillis();
        System.out.println("总计用时"+(endTime - startTime));   //总计用时 940
        bis.close();
        bos.close();
    }

    /**
     * 缓冲区的独特方法
     */
    @Test
    public void t4() throws IOException {
        //  写数据
        BufferedReader br = new BufferedReader(new FileReader("fos.txt"));
        BufferedWriter bw = new BufferedWriter(new FileWriter("fox.java"));
        String str;
        while ((str = br.readLine()) != null) {
            bw.write(str);
            bw.newLine();
        }
        System.out.println("success");
        br.close();
        bw.close();
    }

    @Test
    public void test01() throws IOException {
        //字符
        BufferedReader br = new BufferedReader(new FileReader("fos.txt"));
        BufferedWriter bw = new BufferedWriter(new FileWriter("out.txt"));

        //map集合
        HashMap<Character,Integer> map = new HashMap<>();

        String s;
        while ( (s =br.readLine()) != null ){
            //获取 s的第一个字符
            char name = s.charAt(0);

            if (map.containsKey(name)){
                Integer integer = map.get(name);
                integer++;
                //key Steing  value Integer
                map.put(name,integer);
            }else{
                map.put(name,1);
            }
        }

        //循环map 输出
        Set<Map.Entry<Character, Integer>> entries = map.entrySet();
        //循环输出
        for (Map.Entry<Character, Integer> entry : entries) {
            bw.write(entry.getKey()+":"+entry.getValue());
            bw.newLine();
        }

        bw.close();
        br.close();
    }

    /**
     * 转换流
     */
    @Test
    public void t5() throws IOException {
        FileReader fileReader = new FileReader("d:\\File_GBK.txt");
        int data;
        while ((data = fileReader.read()) != -1){
            System.out.print((char)data);
        }
        fileReader.close();
    }

    @Test
    public void t6() throws IOException {
        InputStreamReader isr = new InputStreamReader(new FileInputStream("d:\\File_GBK.txt"),"UTF-8");
        OutputStreamWriter osw = new OutputStreamWriter(new FileOutputStream("d:\\File_GBK1.txt"),"GBK");
        int data;
        while ((data = isr.read()) != -1){
            System.out.print((char)data);
            osw.write(data);
        }
        isr.close();
        osw.close();
    }


    /**
     * 序列化和反序列化
     */
    @Test
    public void t7() throws IOException {
        String name = "巫师";
        int age = 300;
        char gender = '男';
        int energy = 5000;
        double price = 75.5;
        boolean relive = true;
        //序列化
        ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream("game.dat"));
        oos.writeUTF(name);
        oos.writeInt(age);
        oos.writeChar(gender);
        oos.writeInt(energy);
        oos.writeDouble(price);
        oos.writeBoolean(relive);
        oos.close();
    }


    /**
     * 反序列化
     * @throws IOException
     */
    @Test
    public void t8() throws IOException {
        ObjectInputStream ois = new ObjectInputStream(new FileInputStream("game.dat"));
        String name = ois.readUTF();
        int age = ois.readInt();
        char gender = ois.readChar();
        int energy = ois.readInt();
        double price = ois.readDouble();
        boolean relive = ois.readBoolean();
        System.out.println(name+","+age+","+gender+","+energy+","+price+","+relive);
        ois.close();
    }


    /**
     * 将employee类进行序列化
     */
    @Test
    public void t9() throws IOException{
        Employee employee = new Employee();
        Employee.setCompany("loveCoding");
        employee.setAddress("beijing");
        employee.setAge(18);
        employee.setName("zs");
        //  将对象进行序列化
        ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream("employee.dat"));
        oos.writeObject(employee);
        oos.close();
    }


    /**
     * 将employee类进行反序列化
     */
    @Test
    public void t10() throws IOException, ClassNotFoundException {
        ObjectInputStream ois = new ObjectInputStream(new FileInputStream("employee.dat"));
        Object obj = ois.readObject();
        if(obj instanceof Employee){
            Employee employee = (Employee) obj;
            String company = Employee.getCompany();
            System.out.println(company);
            System.out.println(employee);
        }else
        throw new RuntimeException("反序列化失败");
        ois.close();
    }


    /*
    从键盘输入字符串，
    要求将读取到的整行字符串转成大写输出。
    然后继续进行输入操作，直至当输入“e”或者“exit”时，退出程序
     */
    @Test
    public static void main(String[] args) throws IOException {
        System.out.println("从键盘输入字符串~");
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        String str;
        while ((str = br.readLine()) != null){
            if (str.equals("e")  || str.equals("exit")) {
                break;
            }
            System.out.println("str  - 转大写"+str.toUpperCase());
            }
    }

    @Test
    public void t11() throws FileNotFoundException {
        PrintStream printStream = new PrintStream("fox.txt");
        printStream.print(97);
        printStream.print(10800);
        printStream.close();
    }

    /**
     *   ioUtils
     *    <dependency>
     *       <groupId>commons-io</groupId>
     *       <artifactId>commons-io</artifactId>
     *       <version>2.4</version>
     *     </dependency>
    */
    @Test
    public void t12() throws IOException {
        int copy = IOUtils.copy(new FileInputStream("fox.txt"), new FileOutputStream("fox_bak.txt"));
//        IOUtils.closeQuietly();
        FileUtils.copyDirectory(new File("d:\\demo"),new File("d:\\demobak"));
        FileUtils.writeStringToFile(new File("fox.txt"),"zxcvbbn");
        String s = FileUtils.readFileToString(new File("fox.txt"));
        System.out.println(s);
        FileUtils.copyFile(new File("out.txt"),new File("outbak.txt"));
    }
}
