package com.sf.jdbc.day25;

import org.junit.jupiter.api.Test;

import java.lang.reflect.Field;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;


public class BaseDao<T> {
    /**
     * 封装操作数据库Dao层的通用类
     * <增 删  改></>
     */
    public int update(Connection connection,String sql ,Object ... args) throws SQLException {
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        // 参数绑定
        if(args != null  &&  args.length > 0){
            for (int i = 0;i<args.length;i++){
                preparedStatement.setObject(i+1,args[i]);
            }
        }
        return preparedStatement.executeUpdate();
    }


    /**
     * 查询全部
     * @return
     */
    public List<T> getList(Connection connection,String sql,Class<T> clazz,Object ...args) throws SQLException, InstantiationException, IllegalAccessException, NoSuchFieldException, SecurityException {
        List<T> list = new ArrayList<>();
        T t = null;
        //获取PreparedStatement
        PreparedStatement ps = connection.prepareStatement(sql);
        //赋值
        if(args!=null && args.length>0) {
            for(int i=0;i<args.length;i++) {
                ps.setObject(i+1, args[i]);
            }
        }
        //执行
        ResultSet rs = ps.executeQuery();
        //获取数据库表的元数据信息
        ResultSetMetaData rsmd = rs.getMetaData();
        //获取数据库表的列总数
        int columnCount = rsmd.getColumnCount();
        while(rs.next()) {
            //实例化对象
            t=clazz.newInstance();
            //循环获取列名
            for(int i=0;i<columnCount;i++) {
                //通过列的下标获取列名
                String columnLabel = rsmd.getColumnLabel(i+1);
                //通过列标签获取列值
                Object columnValue = rs.getObject(columnLabel);
                //反射通过名称获取java对象中描述属性的对象
                Field field = clazz.getDeclaredField(columnLabel);//dept_id as deptId
                //取消私有权限
                field.setAccessible(true);
                //给属性对象赋值
                field.set(t, columnValue);
            }
            list.add(t);
        }
        return list;
    }

    /**
     * 查询单个
     * @return
     */
    public T getT(Connection connection,String sql,Class<T> clazz,Object ...args) throws SQLException, InstantiationException, IllegalAccessException, NoSuchFieldException, SecurityException {
        T t = null;
        //获取PreparedStatement
        PreparedStatement ps = connection.prepareStatement(sql);
        //赋值
        if(args!=null && args.length>0) {
            for(int i=0;i<args.length;i++) {
                ps.setObject(i+1, args[i]);
            }
        }
        //执行
        ResultSet rs = ps.executeQuery();
        //获取数据库表的元数据信息
        ResultSetMetaData rsmd = rs.getMetaData();
        //获取数据库表的列总数
        int columnCount = rsmd.getColumnCount();
        if(rs.next()) {
            //实例化对象
            t=clazz.newInstance();
            //循环获取列名
            for(int i=0;i<columnCount;i++) {
                //通过列的下标获取列名
                String columnLabel = rsmd.getColumnLabel(i+1);
                //通过列标签获取列值
                Object columnValue = rs.getObject(columnLabel);
                //反射通过名称获取java对象中描述属性的对象
                Field field = clazz.getDeclaredField(columnLabel);//dept_id as deptId
                //取消私有权限
                field.setAccessible(true);
                //给属性对象赋值
                field.set(t, columnValue);
            }
        }
        return t;
    }
    @Test
    public void t1() throws SQLException, NoSuchFieldException, InstantiationException, IllegalAccessException {
        update(JDBCUtil.getConnection(),"INSERT INTO  user(user_name,price,create_time) values (?,?,?)","qqqqq",123.12,new Date(System.currentTimeMillis()));
        getList(JDBCUtil.getConnection(),"SELECT user_name as userName,price as price,create_time as createTime FROM user", (Class<T>) User.class);

    }
}
